/*
 * Almost an xml beautifier.
 *
 * It can be useful just for re-indenting machine generated xml
 * to make it human readable. It also performs some basic checks
 * on the file's validity which can be handy if hand editing the
 * xml source.
 *
 * It does take some liberties with the data:
 *
 *	- It strips the all comments, though the xml version
 *	   and DOCTYPE "structured comments" are preserved.
 *
 *	- attributes are always quoted with more plan9 style single
 *	  quotes whether the source has single or double quotes.
 *
 *	- empty elements are replaced with the short form, e.g.
 *	   <fred a='1'> </fred> is mapped to <fred a='1' />
 */

#include <u.h>
#include <libc.h>
#include <pool.h>
#include <bio.h>
#include <xml.h>

static void
usage(void)
{
	fprint(2, "usage: %s [-w] [-n] [-h] [file.xml]\n", argv0);
	fprint(2, "  -w   preserve all whitespace\n");
	fprint(2, "  -n   strip namespace specs\n");
	fprint(2, "  -h   do not prepend XML headers to output\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Xml *xp;
	int i, h, flags, header;
	Biobuf *bp, out;

	flags = Fcrushwhite;
	header = 1;
	ARGBEGIN{
	case 'd':
		xml_debug++;
		break;
	case 'w':
		flags &= ~Fcrushwhite;		/* drop whitespace */
		break;
	case 'n':
		flags |= Fstripnamespace;	/* drop namespaces */
		break;
	case 'h':
		header = 0;
		break;
	default:
		usage();
	}ARGEND;


	Binit(&out, 1, OWRITE);
	if(argc == 0){
		do{
			if((xp = xml_parse(0, 8192, flags)) == nil)
				break;
			xml_Bprint(xp, header, &out);
			xml_free(xp);
		}while(xp->root && xp->root->name);
	}
	else
	for(i = 0; i < argc; i++){
		h = header;
		if((bp = Bopen(argv[i], OREAD)) == nil){
			fprint(2, "%s: %s cannot open %r\n", argv0, argv[i]);
			continue;
		}
		do{
			if((xp = xml_Bparse(bp, 8192, flags)) == nil)
				break;
			xml_Bprint(xp, h, &out);
			xml_free(xp);
			h = 0;
		}while(xp->root);
		Bterm(bp);
	}
	Bterm(&out);
	exits(0);
}
